import config, random

class EventManager:
    def __call__(self):
        return self
    
    def __init__(self):
        self.subscribers = []

    def subscribe(self,subscriber):
        self.subscribers.append(subscriber)

    def unsubscribe(self,subscriber):
        if self.subscribers.count(subscriber) > 0:
            self.subscribers.remove(subscriber)

    def notify(self,sender,verb="changed",*args,**kwargs):
        for subscriber in self.subscribers:
            method = "on_%s_%s" % (sender.__class__.__name__.lower(),verb)
            if hasattr(subscriber,method):
                getattr(subscriber,method)(sender,*args,**kwargs)

EventManager = EventManager()

class Agent:
    def __init__(self):
        self.subscribers = []
        self.em = EventManager()
        self.em.subscribe(self)

    def notify(self,verb="changed",*args,**kwargs):
        self.em.notify(self,verb,*args,**kwargs)

    def debug(self,message):
        if config.debug:
            print self.__class__.__name__ + ": " + message

class Engine(Agent):
    def __init__(self, width, height):
    	Agent.__init__(self)
        self.width = width
        self.height = height
        self.updated = True

    def clear(self):
        pass

    def stop(self):
        pass

    def update(self):
        pass

class Creature(Agent):

    def __init__(self,geno,mask=0):
        Agent.__init__(self)
        if geno == None:
            geno = random.getrandbits(8)
        self.geno = geno
        self.hash = int(random.getrandbits(31))

    def __hash__(self):
        return self.hash

    def get_pheno(self):
        return (self.geno >> 6, self.geno >> 4 & 3),(self.geno >> 2 & 3, self.geno & 3)

    def set_pheno(self,pheno):
        self.geno = pheno[1][1] << 6 + pheno[1][0] << 4 + pheno[0][1] << 2 + pheno[0][0]

    def create_random_mask(self, zero=True):
        mask = 3 * random.getrandbits(1) + 12 * random.getrandbits(1) + 48 * random.getrandbits(1) + 192 * random.getrandbits(1)
        if mask == 0L and not zero:
            mask += 3 << (2 * random.randrange(0,4))
        return mask

    pheno = property(get_pheno,set_pheno)


