import config
from base import *

class PygameEngine(Engine):
    def __init__(self, width, height):
        Engine.__init__(self,width,height)
        import pygame
        import pygame.font
        import pygame.time

        self.clock = pygame.time.Clock()
        self.tile_size = 50
        self.phevo_size = 45
        self.bug_size = 45

        self.pygame = pygame
        try:
            flags = pygame.SRCALPHA+pygame.HWSURFACE
            pygame.init()
            self.win = pygame.display.set_mode((self.width, self.height), pygame.RESIZABLE, 32)
            pygame.display.set_caption(config.title)

            self.phevo_surface = pygame.Surface((self.phevo_size * 8, self.phevo_size * 8), flags=flags)
            self.phevo_surface.fill((48,48,48))

            self.opp_phevo_surface = pygame.Surface((self.phevo_size * 8, self.phevo_size * 8), flags=flags)
            self.opp_phevo_surface.fill((48,48,48))

            self.back_surface = pygame.Surface((self.width, self.height), flags=flags)
            #self.back_surface.fill((0,0,0))

            self.innershape_surface = pygame.image.load(config.innershapes)
            self.outershape_surface = pygame.image.load(config.outershapes)

            self.deathicon_surface = pygame.image.load(config.deathicon)
            self.sickicon_surface = pygame.image.load(config.sickicon)

            self.syringe_surface = pygame.Surface((self.bug_size * 5,self.bug_size), flags=flags)
            self.syringe_surface.set_alpha(255)
            self.syringe_surface.fill((192,192,192))

            self.syringeover_surface = pygame.Surface((self.bug_size * 5,self.bug_size), flags=flags)
            self.syringeover_surface.set_alpha(48)
            self.syringeover_surface.fill((0,128,128))

            self.syringe_font = pygame.font.Font(pygame.font.get_default_font(),12)
            self.syringe_font.set_bold(True)

            self.finished = False
            self.clear()
        except Exception, e:
            self.stop()
            raise e

    def __del__(self):
        self.pygame.quit()

    def clear(self):
        self.back_surface.fill((0,0,0))

    def stop(self):
        self.pygame.quit()

    def on_phevopop_populated(self,pop,phevo):
        self.render_phevo(phevo)

    def on_phevo_infected(self,phevo):
        self.render_phevo(phevo)

    def on_phevo_cured(self,phevo):
        self.render_phevo(phevo)

    def on_phevo_died(self,phevo):
        self.render_phevo(phevo)

    def on_syringe_filled(self,syringe,bug):
        if not syringe.player.AI:
            self.syringe_surface.fill((128,128,128))
            for index,bug in enumerate(syringe.bugs):
                right = index * self.bug_size
                self.render_bug(bug,right)

    def on_syringe_sprayed(self,syringe,population,bug):
        if not syringe.player.AI:
            for frame in range(15):
                self.syringe_surface.fill((128,128,128))
                #render the sprayed bug
                right = -3 * frame
                self.render_bug(bug,right)
                #and move the remaining
                for index,next_bug in enumerate(syringe.bugs):
                    right = -3 * frame + (index + 1) * self.bug_size
                    self.render_bug(next_bug,right)
                self.draw()
    
    def render_bug(self,bug,right=0,anim=False):
        #self.syringe_surface.fill((128,128,128))
        if (bug.mask >> 6) & 3 == 3:
            outershape_x = bug.pheno[0][0] * self.tile_size
        else:
            outershape_x = self.tile_size * 4
            
        if (bug.mask >> 4) & 3 == 3:
            outershape_y = bug.pheno[0][1] * self.tile_size
        else:
            outershape_y = self.tile_size * 5
            
        self.syringe_surface.blit(self.outershape_surface,(right,0),(outershape_x,outershape_y,self.tile_size,self.tile_size))
        
        if (bug.mask >> 2) & 3 == 3:
            innershape_x = bug.pheno[1][0] * self.tile_size
        else:
            innershape_x = self.tile_size * 4
            
        if bug.mask & 3 == 3:
            innershape_y = bug.pheno[1][1] * self.tile_size
        else:
            innershape_y = self.tile_size * 5
            
        self.syringe_surface.blit(self.innershape_surface,(right,0),(innershape_x,innershape_y,self.tile_size,self.tile_size))
        
        self.updated = True
        

    def render_phevo(self,phevo):
        outershape_x = phevo.pheno[0][0] * self.tile_size
        innershape_x = phevo.pheno[1][0] * self.tile_size
        outeralpha = inneralpha = 255

        """
        if phevo.infected:
            outershape_y = self.tile_size * 4
            innershape_y = self.tile_size * 4
        elif phevo.dead:
            outershape_y = self.tile_size * 5
            innershape_y = self.tile_size * 5
        else:
            outershape_y = phevo.pheno[0][1] * self.tile_size
            innershape_y = phevo.pheno[1][1] * self.tile_size
        """
        
        icon = None
        if phevo.infected:
            icon = self.sickicon_surface
        elif phevo.dead:
            icon = self.deathicon_surface

        outershape_y = phevo.pheno[0][1] * self.tile_size
        innershape_y = phevo.pheno[1][1] * self.tile_size

        if phevo.population.player.id == 0:
            self.phevo_surface.fill((48,48,48),((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size,self.tile_size,self.tile_size))
            self.phevo_surface.blit(self.outershape_surface,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size),(outershape_x,outershape_y,self.tile_size,self.tile_size))
            self.phevo_surface.blit(self.innershape_surface,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size),(innershape_x,innershape_y,self.tile_size,self.tile_size))
            if icon:
                self.phevo_surface.blit(icon,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size))
        else:
            self.opp_phevo_surface.fill((48,48,48),((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size,self.tile_size,self.tile_size))
            self.opp_phevo_surface.blit(self.outershape_surface,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size),(outershape_x,outershape_y,self.tile_size,self.tile_size))
            self.opp_phevo_surface.blit(self.innershape_surface,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size),(innershape_x,innershape_y,self.tile_size,self.tile_size))
            if icon:
                self.opp_phevo_surface.blit(icon,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size))
            
        self.updated = True
        self.debug("rendering %s" % phevo)

    def tick(self):
        self.notify("ticked")
        self.clock.tick(60)

    def poll_controller(self):    
        for event in self.pygame.event.get():
            if event.type == self.pygame.QUIT:
                self.finished = True
                self.notify()
            elif event.type == self.pygame.KEYDOWN:
                if event.key in (self.pygame.K_q, self.pygame.K_ESCAPE):
                    self.finished = True
                    self.notify()
                elif event.key == self.pygame.K_s:
                    self.notify("spray_requested",0)

    def draw(self):
        if self.updated:
            self.win.blit(self.back_surface,(0,0))
            self.win.blit(self.phevo_surface,(0,0))
            self.win.blit(self.opp_phevo_surface,(self.phevo_surface.get_width() + config.margin,0))
            self.win.blit(self.syringe_surface,(0,self.phevo_surface.get_height() + config.margin))
            self.win.blit(self.syringeover_surface,(0,self.phevo_surface.get_height() + config.margin))
            self.pygame.display.flip()
            self.clear()
            self.updated = False

    def on_pygameengine_ticked(self,engine):
        self.poll_controller()
        self.draw()

